@@ -1,20 +1,29 @@
+# -*- coding: utf-8 -*-
 import datetime
+import logging
 import os
 import socket
-
-from sqlalchemy import or_
-from sqlalchemy.sql.expression import asc
-from sqlalchemy.sql.expression import desc
-from sqlalchemy import create_engine
-from sqlalchemy.exc import OperationalError, ProgrammingError
-from sqlalchemy.orm import sessionmaker
-from uuid import uuid4
+import urllib
 
 from cronq.config import DATABASE_URL
 from cronq.models.category import Category
 from cronq.models.event import Event
 from cronq.models.job import Job
 
+from sqlalchemy import create_engine
+from sqlalchemy import or_
+from sqlalchemy.exc import InternalError
+from sqlalchemy.exc import OperationalError
+from sqlalchemy.exc import ProgrammingError
+from sqlalchemy.orm import sessionmaker
+from sqlalchemy.sql.expression import asc
+from sqlalchemy.sql.expression import desc
+
+from uuid import uuid4
+
+
+logger = logging.getLogger(__name__)
+
 
 class Storage(object):
 
@@ -23,24 +32,26 @@ class Storage(object):
     STARTED = 'started'
     SUCCEEDED = 'succeeded'
 
-    def __init__(self, publisher=None):
+    def __init__(self, publisher=None, isolation_level='SERIALIZABLE'):
         self.publisher = publisher
 
-        self._engine = self._new_engine()
+        self._engine = self._new_engine(isolation_level=isolation_level)
         self._maker = sessionmaker(bind=self._engine)
         self.session = self._new_session()
 
-    def _new_engine(self):
-        return create_engine(DATABASE_URL, isolation_level='SERIALIZABLE')
+    def _new_engine(self, isolation_level='SERIALIZABLE'):
+        if isolation_level is not None:
+            return create_engine(DATABASE_URL, isolation_level=isolation_level)
+        return create_engine(DATABASE_URL)
 
     def _new_session(self):
         return self._maker()
 
     def bootstrap(self):
         models = [
+            Category,
             Job,
             Event,
-            Category,
         ]
         for model in models:
             try:
@@ -90,7 +101,19 @@ class Storage(object):
         self.session.add(event)
         self.session.commit()
 
-    def update_job_status(self, job_id, _datetime, status, return_code=None):
+    def update_job_status(self, run_id, job_id, _datetime, status, return_code=None):
+        while True:
+            try:
+                self._update_job_status(job_id, _datetime, status, return_code)
+                break
+            except InternalError:
+                logger.info('[cronq_job_id:{0}] [cronq_run_id:{1}] Unable to update job with result, retrying'.format(
+                    job_id, run_id
+                ))
+                self.session.rollback()
+        return True
+
+    def _update_job_status(self, job_id, _datetime, status, return_code=None):
         job = self.session.query(Job).filter_by(id=job_id).first()
         if job:
             job.current_status = status
@@ -176,16 +199,26 @@ class Storage(object):
         if len(events) == 0:
             return []
 
+
         chunks = {}
         for event in events:
             if event['run_id'] not in chunks:
-                chunks[event['run_id']] = {'first': None, 'last': None}
+
+                chunks[event['run_id']] = {
+                    'first': None,
+                    'last': None,
+                    'job_id': event['job_id']
+                }
 
             if event['type'] == 'starting':
                 chunks[event['run_id']]['first'] = event
             else:
                 chunks[event['run_id']]['last'] = event
 
+        log_url_template = os.getenv('CRONQ_LOG_URL_TEMPLATE', None)
+        if log_url_template:
+            self._add_log_urls(chunks, log_url_template)
+
         docs = [chunk for i, chunk in chunks.iteritems()]
 
         def chunk_compare(x, y):
@@ -199,6 +232,7 @@ class Storage(object):
                 return 1
             return int((first['datetime'] - last['datetime']).total_seconds())
 
+        # newest to oldest
         docs = sorted(docs, cmp=chunk_compare, reverse=True)
 
         return docs
@@ -222,41 +256,88 @@ class Storage(object):
 
     def inject(self):
         """Get a message from storage and injects it into the job stream"""
-        print 'Trying to inject', datetime.datetime.utcnow()
+        logger.info('Trying to inject')
         while self.get_unpublished_task() is not None:
             pass
 
-    def get_unpublished_task(self):
-        session = self._new_session()
+    def get_job_to_inject(self, session):
         to_run = or_(
             Job.next_run < datetime.datetime.utcnow(),
-            Job.run_now == True,
+            Job.run_now == True,  # noqa
         )
-
         job = session.query(Job).filter(to_run).first()
         if job is None:
+            return None
+
+        logger.info('[cronq_job_id:{0}] Found a job: {1} {2}'.format(
+            job.id, job.name, job.next_run))
+
+        return job
+
+    def update_job_time(self, session, job):
+        if job.next_run is None:
+            current_time = datetime.datetime.utcnow()
+            logger.info('[cronq_job_id:{0}] Setting time to {1}'.format(job.id, current_time))
+            job.next_run = current_time
+        else:
+            while job.next_run < datetime.datetime.utcnow():
+                logger.info('[cronq_job_id:{0}] Adding time!'.format(job.id))
+                job.next_run += job.interval
+        job.run_now = False
+        me = '{0}.{1}'.format(socket.gethostname(), os.getpid())
+        job.locked_by = me
+
+        # update
+        try:
+            session.merge(job)
+            session.commit()
+        except InternalError, e:
+            session.rollback()
+            logger.warning('[cronq_job_id:{0}] Error updating time {1} - {2}'.format(
+                job.id, job.name, e))
+            return None
+        except Exception, e:
+            session.rollback()
+            logger.exception('[cronq_job_id:{0}] {1} {2}'.format(job.id, job.name, e))
+            raise
+        else:
+            logger.info('[cronq_job_id:{0}] Next job run: {1}'.format(job.id, job.next_run))
+            return job
+
+    def get_unpublished_task(self):
+        session = self._new_session()
+
+        job = self.get_job_to_inject(session)
+        if not job:
+            logger.info("no job found")
             session.close()
             return
-        print 'Found a job:', job.name, job.next_run
 
-        while job.next_run < datetime.datetime.utcnow():
-            print 'Adding time!'
-            job.next_run += job.interval
-        print job.next_run
+        # update job time
+        job = self.update_job_time(session, job)
+        if not job:
+            logger.info("no job found after update time")
+            session.close()
+            return
+
+        # inject
         job_doc = {
             'name': job.name,
             'command': unicode(job.command),
             'id': job.id,
         }
-        job.run_now = False
-        me = '{0}.{1}'.format(socket.gethostname(), os.getpid())
-        job.locked_by = me
-        try:
-            session.commit()
-            self.publisher.publish(job.routing_key, job_doc, uuid4().hex)
-        except Exception:
-            session.rollback()
-            raise
+
+        # publish
+        logger.info("[cronq_job_id:{0}] Trying to publish job".format(job.id))
+        success = self.publisher.publish(job.routing_key, job_doc, uuid4().hex)
+        if not success:
+            logger.warning('[cronq_job_id:{0}] Error publishing: {1}'.format(
+                job.id, job.name))
+            session.close()
+            return
+
+        logger.info("[cronq_job_id:{0}] Job published".format(job.id))
+
         session.close()
         return True
 
@@ -284,3 +365,31 @@ class Storage(object):
                 return self.SUCCEEDED
 
         return self.FINISHED
+
+
+    @staticmethod
+    def _add_log_urls(chunks, log_url_template):
+        for run_id, run_data in chunks.items():
+            log_url = log_url_template \
+                .replace('{job_id}', str(run_data['job_id'])) \
+                .replace('{run_id}', str(run_id))
+
+            if any([time_field in log_url_template for time_field in ('{start_time}', '{end_time}')]):
+                if run_data.get('first'):
+                    start_time = urllib.quote(run_data['first']['datetime'].strftime('%Y-%m-%dT%H:%M:%S.000Z'))
+                else:
+                    # default to now minus 24hrs if field is missing. this generally means bad data
+                    logger.warning("No start time found for {0}, using 24 hours ago for log url".format(run_id))
+                    one_day_ago = datetime.datetime.utcnow() - datetime.timedelta(hours=24)
+                    start_time = urllib.quote(one_day_ago.strftime('%Y-%m-%dT%H:%M:%S.000Z'))
+
+                if run_data.get('last'):
+                    end_time = urllib.quote(run_data['last']['datetime'].strftime('%Y-%m-%dT%H:%M:%S.999Z'))
+                else:
+                    # default to just slightly in the future if not present. probably means still running
+                    few_seconds_in_future = datetime.datetime.utcnow() + datetime.timedelta(seconds=5)
+                    end_time = urllib.quote(few_seconds_in_future.strftime('%Y-%m-%dT%H:%M:%S.000Z'))
+                log_url = log_url.replace('{start_time}', start_time)
+                log_url = log_url.replace('{end_time}', end_time)
+
+            run_data['log_url'] = log_url
