@@ -1,3 +1,5 @@
+# -*- coding: utf-8 -*-
+import logging
 import time
 
 
@@ -16,7 +18,12 @@ class Injector(object):
 def main():
     from cronq.backends.mysql import Storage
     from cronq.queue_connection import Publisher
+
+    logger = logging.getLogger('cronq.injector')
+    logger.info('Creating injector')
     injector = Injector(Storage(Publisher()))
+
+    logger.info('Running injector')
     injector.run()
 
 
