@@ -1,24 +1,30 @@
-#!/usr/bin/env python
-#-*- coding:utf-8 -*-
+# -*- coding: utf-8 -*-
 import datetime
-import gevent
-import json
+import fcntl
 import importlib
+import json
 import logging
 import logging.handlers
 import os
+import random
+import re
 import socket
 import subprocess
 import sys
 import time
 
-from haigha.message import Message
-
 from cronq.config import LOG_PATH
 from cronq.config import QUEUE
 from cronq.queue_connection import connect
+from cronq.utils import unicodedammit
 
-logger = logging.getLogger('cronq')
+import gevent
+
+from haigha.message import Message
+
+logger = logging.getLogger(__name__)
+
+FILENAME_REGEX = re.compile('[\W_]+', re.UNICODE)
 
 
 class NullHandler(logging.Handler):
@@ -29,18 +35,15 @@ logger.addHandler(NullHandler())
 
 
 def channel_closed_cb(ch):
-    message = "AMQP channel closed; close-info: {0}"
-    print message.format(ch.close_info)
-    ch = None
-    return
+    message = "AMQP channel closed; close-info: {0}".format(ch.close_info)
+    logger.warning(message)
 
 
 def create_connection_closed_cb(connection):
     def connection_closed_cb():
-        message = "AMQP broker connection closed; close-info: {0}"
-        print message.format(connection.close_info)
-        connection = None
-        connection
+        message = "AMQP broker connection closed; close-info: {0}".format(
+            connection.close_info)
+        logger.warning(message)
     return connection_closed_cb
 
 
@@ -75,7 +78,7 @@ def make_directory(directory):
         pass
 
 
-def create_runner(channel):
+def create_runner(channel):  # noqa
 
     def run_something(msg):
         tag = msg.delivery_info['delivery_tag']
@@ -101,7 +104,7 @@ def create_runner(channel):
             valid = True
             for key in ['cmd', 'job_id', 'run_id']:
                 if not data.get(key, None):
-                    print 'Missing {0}'.format(key)
+                    logger.debug('Missing {0}'.format(key))
                     valid = False
             return valid
 
@@ -110,7 +113,9 @@ def create_runner(channel):
             return reject(requeue=False)
 
         cmd = data.get('cmd')
-        print 'starting'
+        logger.info('[cronq_job_id:{0}] [cronq_run_id:{1}] Starting {2}'.format(
+            data.get('job_id'), data.get('run_id'), cmd
+        ))
         publish_result({
             'job_id': data.get('job_id'),
             'run_id': data.get('run_id'),
@@ -118,16 +123,18 @@ def create_runner(channel):
             'type': 'starting',
         })
         start = time.time()
+        process = None
         try:
-            proc = subprocess.Popen(
+            process = subprocess.Popen(
                 cmd,
                 shell=True,
                 stdout=subprocess.PIPE,
                 stderr=subprocess.STDOUT,
-                bufsize=1,
             )
-        except OSError as exc:
-            print 'Failed', exc
+        except OSError:
+            logger.exception('[cronq_job_id:{0}] [cronq_run_id:{1}] Failed job'.format(
+                data.get('job_id'), data.get('run_id')
+            ))
             end = time.time()
             publish_result({
                 'job_id': data.get('job_id'),
@@ -136,28 +143,69 @@ def create_runner(channel):
                 'type': 'failed',
             })
             return reject(requeue=False)
-        print 'waiting'
 
-        filename = '{0}/{1}.log'.format(LOG_PATH, data.get('name', 'UNKNOWN'))
+        fd = process.stdout.fileno()
+        fl = fcntl.fcntl(fd, fcntl.F_GETFL)
+        fcntl.fcntl(fd, fcntl.F_SETFL, fl | os.O_NONBLOCK)
+
+        logger.info('[cronq_job_id:{0}] [cronq_run_id:{1}] Waiting'.format(
+            data.get('job_id'), data.get('run_id')
+        ))
+
+        splits = FILENAME_REGEX.split(data.get('name', 'UNKNOWN'))
+        if len(splits) > 1:
+            logfile = '-'.join(splits)
+        filename = '{0}/{1}.log'.format(LOG_PATH, logfile.strip('-'))
+
         handler = logging.handlers.WatchedFileHandler(filename)
-        while proc.returncode is None:
-            log_record = logging.makeLogRecord({
-                'msg': proc.stdout.read(1024),
-            })
-            handler.emit(log_record)
-            proc.poll()
+        log_to_stdout = bool(os.getenv('CRONQ_RUNNER_LOG_TO_STDOUT', False))
+
+        while True:
+            try:
+                nextline = process.stdout.readline()
+            except IOError:
+                nextline = ''
+
+            if nextline == '' and process.poll() is not None:
+                break
+
+            if nextline == '':
+                time.sleep(0.1 * random.random())
+                continue
+
+            try:
+                message = nextline.rstrip()
+                message = unicodedammit(message)
+            except:
+                continue
+
+            if message:
+                for m in message.splitlines():
+                    log_record = logging.makeLogRecord({
+                        'msg': message,
+                    })
+                    handler.emit(log_record)
+                    if log_to_stdout:
+                        logger.info(u'[cronq_job_id:{0}] [cronq_run_id:{1}] {2}'.format(
+                            data.get('job_id'), data.get('run_id'), log_record.getMessage()
+                        ))
+
+            time.sleep(0.00001)
+            sys.stdout.flush()
+
         handler.close()
 
         end = time.time()
         publish_result({
             'job_id': data.get('job_id'),
             'run_id': data.get('run_id'),
-            'return_code': proc.returncode,
+            'return_code': process.returncode,
             'run_time': end - start,
             'type': 'finished',
         })
-        print 'done', proc.returncode, data.get('run_id')
-
+        logger.info('[cronq_job_id:{0}] [cronq_run_id:{1}] [cronq_exit_code:{2}] Done'.format(
+            data.get('job_id'), data.get('run_id'), process.returncode
+        ))
         ack()
 
     return run_something
