@@ -1,23 +1,23 @@
+# -*- coding: utf-8 -*-
 import datetime
+import json
 import logging
 
-import json
-
-from flask import Flask
-from flask import g
-from flask import render_template
-from flask import request
-from flask import Response
-from flask import redirect
-from flask import url_for
-from flask import flash
-from flask import abort
-
 from cronq import interval_parser
+from cronq.backends.mysql import Storage
 from cronq.utils import split_command
 from cronq.utils import task_status
 from cronq.utils import took
-from cronq.backends.mysql import Storage
+
+from flask import Flask
+from flask import Response
+from flask import abort
+from flask import flash
+from flask import g
+from flask import redirect
+from flask import render_template
+from flask import request
+from flask import url_for
 
 app = Flask(__name__)
 stream_handler = logging.StreamHandler()
@@ -28,12 +28,14 @@ app.jinja_env.filters['split_command'] = split_command
 app.jinja_env.globals.update(task_status=task_status)
 app.jinja_env.globals.update(took=took)
 
+logger = logging.getLogger(__name__)
+
 
 @app.before_request
 def create_storage():
     if request.path.startswith('/static/'):
         return
-    g.storage = Storage()
+    g.storage = Storage(isolation_level=None)
 
 
 @app.after_request
@@ -41,8 +43,8 @@ def remove_storage(request):
     if hasattr(g, 'storage'):
         try:
             g.storage.close()
-        except Exception as exc:
-            print exc
+        except Exception:
+            logger.exception("exception in remove storage")
     return request
 
 
